--[[
A UnitGroupControl is a single button that is associated with a unit frame.
When the appropriate button combination is clicked on the unit frame, the unit
frame sends a Click message to the associated button.  That button then
transitions state, which signals the associated UnitGroupHeader, changing the
state of the header.  This in turn propogates to all ActionButton and
ActionButtonNone instances associated with that header, causing those buttons
to show.

There is one UnitGroupControl associated with each unit frame that registers.
With WoW 2.01, each registered frame gets a distinct UnitGroupControl instance.
With WoW 2.02, a registered frame gets the UnitGroupControl instance associated
with the unit the frame is associated with.  If the unit is not associated with
a frame, it reverts to the WoW 2.0.1 behavior.

Each UnitGroupControl references a single UnitGroupHeader.
--]]
uab.classes.UnitGroupControl = uab.mixins.aceOO.Class()

function uab.classes.UnitGroupControl.prototype:init(header, state, unit)
    uab.classes.UnitGroupControl.super.prototype.init(self)

	self.header = header
	self.state = state
	self.unit = unit
	self.enabled = true

	--[[
	Create the control frame
	--]]
    self.frame = CreateFrame("Button", self.header.name.."Control"..self.state, nil, "SecureAnchorButtonTemplate")

    self.frame:SetAttribute("childreparent", true)
    self.frame:SetAttribute("childstate", "^"..self.state)
    
    self.frame.uabcontrol = self

    self:SecureHeaderChanged()
    self:FastChangeUnitChanged()
    
    self.frame:SetAttribute("useparent-unit", true)
    self.frame:SetAttribute("useparent-unitsuffix", true)
end

function uab.classes.UnitGroupControl.prototype:SecureHeaderChanged()
    if self.header.secureheader then
        self.frame:SetAttribute("anchorchild", self.header.secureheader.frame)
        self:FastChangeUnitChanged()
        self:PositionChanged()
        self:ScaleChanged()
    end
end

function uab.classes.UnitGroupControl.prototype:FastChangeUnitChanged()
    if self.header.secureheader then
        if not uab.db.profile.fastchangeunit then
            self.header.secureheader:SetAnchorState(self.state, "0:$input;"..tostring(self.state)..":;*:0")
        else
            self.header.secureheader:SetAnchorState(self.state, tostring(self.state)..":;*:$input")
        end
    end
end

function uab.classes.UnitGroupControl.prototype:Enable()
	if self.unitframe and not self:ActiveOnFrame() then
        --[[
		Indicate that we do not disable this frame directly.  It may, however,
        be disabled by the frame group for the associated frame.
		--]]
		self.enabled = true
        self:SetEnabledState()
	end
end

function uab.classes.UnitGroupControl.prototype:Disable()
	if self:ActiveOnFrame() or self.enabled then
		self.enabled = nil
        self:SetEnabledState()
	end
end

function uab.classes.UnitGroupControl.prototype:PositionChanged()
    if self:ActiveOnFrame() then
        local frameinfo = self:GetFrameGroupInfo()
        
        if frameinfo and not frameinfo.usedefaultanchor then
            uab.main.secureheader:SetStatePosition(
                self.state,
                frameinfo.x or uab.db.profile.position.default.x or 0,
                frameinfo.y or uab.db.profile.position.default.y or 0,
                frameinfo.relpoint or uab.db.profile.position.default.relpoint or "cursor",
                frameinfo.point or uab.db.profile.position.default.point or "CENTER")
        else
            uab.main.secureheader:SetStatePosition(self.state)
        end
    else
        uab.main.secureheader:SetStatePosition(self.state)
    end
end

function uab.classes.UnitGroupControl.prototype:ScaleChanged()
    if self:ActiveOnFrame() then
        local frameinfo = self:GetFrameGroupInfo()
        
        if frameinfo and not frameinfo.usedefaultscale then
            local scale = frameinfo.scale or uab.db.profile.scale.default.value or 1.00
            uab.main.secureheader:SetStateScale(self.state, scale)
        else
            uab.main.secureheader:SetStateScale(self.state)
        end
    else
        uab.main.secureheader:SetStateScale(self.state)
    end
end

function uab.classes.UnitGroupControl.prototype:SwitchToDirectAction(action)
    if self:ActiveOnFrame() then
        self:ClearActivator()
    end

    self.directaction = action
    
    if self:ActiveOnFrame() then
        self:SetActivator()
    end
end

function uab.classes.UnitGroupControl.prototype:SwitchToClickAction()
    if self:ActiveOnFrame() then
        self:ClearActivator()
    end

    self.directaction = nil
    
    if self:ActiveOnFrame() then
        self:SetActivator()
    end
end

function uab.classes.UnitGroupControl.prototype:AssignFrame(frame)
    self:UnassignFrame()
    
    self.unitframe = frame
    self.unitframeunit = SecureButton_GetModifiedUnit(frame)
    self.frame:SetParent(self.unitframe)

    if self:Activatable() then
		self:SetActivator()
	end
end

function uab.classes.UnitGroupControl.prototype:UnassignFrame()
    if self:ActiveOnFrame() then
        self:ClearActivator()
    end
    
    self.frame:SetParent(UIParent)
    self.unitframe = nil
    self.unitframeunit = nil
end

function uab.classes.UnitGroupControl.prototype:IsRegisteredFrame(frame)
    if self.unitframe == frame then
        return true
    end
end

function uab.classes.UnitGroupControl.prototype:ActiveOnFrame()
    return self.unitframe and self:Activatable()
end

function uab.classes.UnitGroupControl.prototype:Activatable()
    return self.activatable
end

function uab.classes.UnitGroupControl.prototype:SetEnabledState()
    --[[
    This method is called when the enabled/disabled state changes for
    this control or for the frame group of the frame associated with
	this control.  If the frame does not match any frame group, it follows
	the enabled for the control.  That is, if either the control is disabled
	directly or the controls frame is in a frame group that is disabled, then
    the control is disabled.  Otherwise, the frame is enabled.
    --]]
    local active = self:ActiveOnFrame()
    local activatable = self.enabled
    
    if activatable and self.unitframe then
        local frameinfo = self:GetFrameGroupInfo()
        
        if frameinfo then
            if frameinfo.disabled then
                activatable = nil
            end
        end
    else
        activatable = nil
    end

    self.activatable = activatable

    if self.activatable and not active then
        self:SetActivator()
    elseif not self.activatable and active then
        self:ClearActivator()
    end
end

function uab.classes.UnitGroupControl.prototype:GetScale()
    return self.header.group:GetScale()
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.UnitGroupControl.prototype:GetFrameGroupInfo()
    local myname = self.unitframe:GetName()
    
    if not self.framegroup then
        for name, info in pairs(uab.db.profile.framegroups) do
            local exp = info.expression
            
            if exp then
                if string.find(myname, exp) then
                    self.framegroup = info
                    break
                end
            end
        end
    end
    
    return self.framegroup
end

function uab.classes.UnitGroupControl.prototype:ClearActivator()
    if self.savedattrs then
        for name, val in pairs(self.savedattrs) do
            self.unitframe:SetAttribute(name, val.oldval)
        end
        
        self.savedattrs = {}
    end
end

function uab.classes.UnitGroupControl.prototype:SetActivator()
    self.savedattrs = {}
    
    if self.directaction then
        --[[
        These actions contain no modifiers.  We need to add our prefix and postfix to them before setting the values
        --]]        
        local attrs = self.directaction:GetData():GetAttributes()
        
    	for name, val in pairs(attrs) do
    		self.savedattrs[self.header.group.activator.prefix..name..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix..name..self.header.group.activator.buttoncode) }
    		self.unitframe:SetAttribute(self.header.group.activator.prefix..name..self.header.group.activator.buttoncode, val)
    	end
    else
        if not self.savedattrs then
            self.savedattrs = {}
        end
    
        self.savedattrs[self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode) }
        self.savedattrs[self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode] = { oldval = self.unitframe:GetAttribute(self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode) }

        self.unitframe:SetAttribute(self.header.group.activator.prefix.."type"..self.header.group.activator.buttoncode, "click")
        self.unitframe:SetAttribute(self.header.group.activator.prefix.."clickbutton"..self.header.group.activator.buttoncode, self.frame)
    end
end

